#undef REQUIRE_PLUGIN
#include <sourceirc>

#define PLUGIN_VERSION "1.2.0"

new Handle:kv_items = INVALID_HANDLE;
new Handle:v_Debug = INVALID_HANDLE;
new Handle:v_Timestamp = INVALID_HANDLE;
new Handle:v_Timeoffset = INVALID_HANDLE;
new Handle:v_ShowSteamID = INVALID_HANDLE;
new Handle:v_HideFakeItems = INVALID_HANDLE;


public Plugin:myinfo = {
	name = "[TF2] SourceIRC -> Item Bot",
	author = "DarthNinja",
	description = "Relays found items",
	version = PLUGIN_VERSION,
	url = "DarthNinja.com"
};

public OnPluginStart()
{
	CreateConVar("sm_itembotlite_version", PLUGIN_VERSION, "Version", FCVAR_PLUGIN|FCVAR_SPONLY|FCVAR_REPLICATED|FCVAR_NOTIFY);
	v_Debug = CreateConVar("sm_itembot_debug", "0", "Enable/Disable debug mode", 0, true, 0.0, true, 1.0);
	v_Timestamp = CreateConVar("sm_itembot_timestamp", "1", "Show/Hide server timestamps of when an item is found", 0, true, 0.0, true, 1.0);
	v_ShowSteamID = CreateConVar("sm_itembot_showsteams", "1", "Show/Hide user's steamid when an item is found", 0, true, 0.0, true, 1.0);
	v_Timeoffset = CreateConVar("sm_itembot_timeoffset", "0", "Seconds to change timestamp by");
	v_HideFakeItems = CreateConVar("sm_itembot_hidefakes", "1", "Set to 0 and fake item found events will also be shown", 0, true, 0.0, true, 1.0);

	LoadTranslations("common.phrases");
	LoadTranslations("plugin.basecommands");

	HookEvent("item_found", OnItemFound);

	RegAdminCmd("sm_reloaditembot", ReloadCFGs, ADMFLAG_ROOT);
	LoadConfigs()

	IRC_MsgFlaggedChannels("items", "ItemBotLite version %s online!", PLUGIN_VERSION);
}

public Action:ReloadCFGs(client, args)
{
	LoadConfigs()

	ReplyToCommand(client, "ItemBot: Reloaded Configs");
	return Plugin_Handled;
}

LoadConfigs()
{
	decl String:tfenglish[512];
	BuildPath(Path_SM, tfenglish, sizeof(tfenglish), "configs/ItemBot_Items.kv");
	kv_items = CreateKeyValues("items");
	if (!FileToKeyValues(kv_items, tfenglish))
	{
		SetFailState("Error: item data not found or cannot be read (File: configs/ItemBot_Items.kv)");
	}

	AutoExecConfig(true, "ItemBot");
}

public OnAllPluginsLoaded()
{
	if (LibraryExists("sourceirc"))
	{
		IRC_Loaded();
	}
}

public OnLibraryAdded(const String:name[])
{
	if (StrEqual(name, "sourceirc", false))
	{
		IRC_Loaded();
	}
}

IRC_Loaded()
{
	IRC_CleanUp();
	//IRC_RegAdminCmd("kick", Command_Kick, ADMFLAG_KICK, "kick <#userid|name> [reason] - Kicks a player from the server");
	IRC_RegCmd("backpack", LinkBackpack, "backpack <#userid|name|steamid> - Gets a user's backpack link.");
	IRC_RegCmd("bp", LinkBackpack, "backpack <#userid|name|steamid> - Gets a user's backpack link.");

}


public Action:LinkBackpack(const String:nick[], args)
{
	if (args != 1)
	{
		IRC_ReplyToCommand(nick, "Usage: backpack <#userid|name|steamid>");
		return Plugin_Handled;
	}

	decl String:sTarget[128];
	IRC_GetCmdArg(1, sTarget, sizeof(sTarget));

	new target = FindTarget(0, sTarget, true, false)

	decl String:SteamID[128];
	new iTargetType;
	if (target != -1)
	{
		GetClientAuthString(target, SteamID, sizeof(SteamID));
		iTargetType = 1; //Client
	}
	else if (StrContains(sTarget, "STEAM_0:", false) != -1)
	{
		strcopy(SteamID, sizeof(SteamID), sTarget);
		iTargetType = 2; //SteamID
	}
	else if (StrContains(sTarget, "76561") != -1)
	{
		iTargetType = 3; //Lazy user putting in a profile url (wtf?)
	}
	else //Assume it's a lazy user putting in a custom url
	{
		iTargetType = 4; //Lazy user putting in a profile url (wtf?)
	}

	decl String:CommunityID[128];
	if (iTargetType == 1 || iTargetType == 2)
	{

		//'Key' number 76561197960265728
		//Break into 765611979 and 60265728

		//Work out community id
		new String:buffer[3][32];
		ExplodeString(SteamID, ":", buffer, 3, 32);

		new iAuthServer = StringToInt(buffer[1]);
		new iClientAuth = StringToInt(buffer[2]);

		new iSCID = (iClientAuth * 2) + iAuthServer + 60265728; //Work out client's Steam Community ID (SCID) 'code'

		if (iSCID >= 100000000) //Would carry over, so fixit
		{
			decl String:sNum[2];
			IntToString(iSCID, sNum, 2);
			FormatEx(sNum, sizeof(sNum), "%s", sNum);
			new result = 765611979 + StringToInt(sNum);
			iSCID = iSCID-100000000;

			if (iSCID < 10000000)
			{
				FormatEx(CommunityID, sizeof(CommunityID), "%i%i%i", result, 0, iSCID);
			}
			else
			{
				FormatEx(CommunityID, sizeof(CommunityID), "%i%i", result, iSCID);
			}
		}
		else
		{
			FormatEx(CommunityID, sizeof(CommunityID), "765611979%i", iSCID);
		}
	}

	decl String:TF2b[128];
	decl String:TF2i[128];
	decl String:Message[256];

	if (iTargetType == 1) //SteamID - In Server
	{
		FormatEx(TF2b, sizeof(TF2b), "http://tf2b.com/?id=%s", CommunityID);
		FormatEx(TF2i, sizeof(TF2i), "http://www.tf2items.com/profiles/%s", CommunityID);

		new team = IRC_GetTeamColor(GetClientTeam(target));
		FormatEx(Message, sizeof(Message), "\x03%02d%N's\x03 backpack: %s", team, target, TF2b);
	}
	else if (iTargetType == 2) // SteamID - Not in server
	{
		FormatEx(TF2b, sizeof(TF2b), "http://tf2b.com/?id=%s", CommunityID);
		FormatEx(TF2i, sizeof(TF2i), "http://www.tf2items.com/profiles/%s", CommunityID);

		FormatEx(Message, sizeof(Message), "Backpack for \x0306%s\x03: %s", SteamID, TF2b);
	}
	else if (iTargetType == 3) // Profile ID - Lazy User(?)
	{
		FormatEx(TF2b, sizeof(TF2b), "http://tf2b.com/?id=%s", sTarget);
		FormatEx(TF2i, sizeof(TF2i), "http://www.tf2items.com/profiles/%s", sTarget);

		FormatEx(Message, sizeof(Message), "%s", TF2b);
	}
	else if (iTargetType == 4)
	{
		FormatEx(TF2b, sizeof(TF2b), "http://tf2b.com/?id=%s", sTarget);
		FormatEx(TF2i, sizeof(TF2i), "http://www.tf2items.com/id/%s", sTarget);

		FormatEx(Message, sizeof(Message), "%s", TF2b);
	}
	else
	{
		IRC_ReplyToCommand(nick, "This message is unpossible");
	}

	IRC_ReplyToCommand(nick, Message);

	return Plugin_Handled;
}


public Action:OnItemFound(Handle:hEvent, const String:strName[], bool:bDontBroadcast)
{
	if (GetEventBool(hEvent, "isfake") && GetConVarBool(v_HideFakeItems))	//Item is fake and we should hide it
		return;

	new client = GetEventInt(hEvent, "player");

	decl String:ircMessage[IRC_MAXLEN];
	decl String:message[256];
	ircMessage[0] = '\0';

	new iItemIndex = GetEventInt(hEvent, "itemdef");
	new itemQuality = GetEventInt(hEvent, "quality");
	new iMethod = GetEventInt(hEvent, "method");

	new team = -1;
	if (client != 0)
		team = IRC_GetTeamColor(GetClientTeam(client));
	if (team == -1)
		Format(message, sizeof(message), "\x03[\x0300 %N \x03\x03]: ", client);
	else
		Format(message, sizeof(message), "\x03[\x03%02d %N \x03]: ", team, client);
	StrCat(ircMessage, sizeof(ircMessage), message);

	/*
	##############
	# IRC Colors #
	##############

	\x03 = Color Char
	%02d = Color
	0 = White
	1 = Black
	2 = Dark Blue
	3 = Dark Green
	4 = Red
	5 = Dark Red
	6 = Purple
	7 = Orange
	8 = Yellow
	9 = Bright Green
	10 = Gray-Blue
	11 = Sky Blue
	12 = Mid Blue
	13 = Pink
	14 = Gray
	15 = White-Grey

	*/

	new irc_itemQuality;
	decl String:str_QualityPrefix[56];
	switch (itemQuality) //Translate IRC colors.  If none are close, use 9 for bright green
	{
		case 0:
		{
			irc_itemQuality = 14;
			Format(str_QualityPrefix, sizeof(str_QualityPrefix), "");  //Unused
		}
		case 1:
		{
			irc_itemQuality = 3;
			Format(str_QualityPrefix, sizeof(str_QualityPrefix), "Genuine ");
		}
		case 2:
		{
			irc_itemQuality = 9;
			Format(str_QualityPrefix, sizeof(str_QualityPrefix), "");  //Unused - ingame color olive
		}
		case 3:
		{
			irc_itemQuality = 12;
			Format(str_QualityPrefix, sizeof(str_QualityPrefix), "Vintage ");
		}
		case 4:
		{
			irc_itemQuality = 7;
			Format(str_QualityPrefix, sizeof(str_QualityPrefix), "");  //Unused - ingame color orange
		}
		case 5:
		{
			irc_itemQuality = 6;
			Format(str_QualityPrefix, sizeof(str_QualityPrefix), "Unusual ");
		}
		case 6:
		{
			irc_itemQuality = 8;
			Format(str_QualityPrefix, sizeof(str_QualityPrefix), "");  //Normal - no prefix
		}
		case 7:
		{
			irc_itemQuality = 3;
			Format(str_QualityPrefix, sizeof(str_QualityPrefix), "Community ");
		}
		case 8:
		{
			irc_itemQuality = 13;
			Format(str_QualityPrefix, sizeof(str_QualityPrefix), "Valve ");
		}
		case 9:
		{
			irc_itemQuality = 3;
			Format(str_QualityPrefix, sizeof(str_QualityPrefix), "Self-Made ");
		}
		case 10:
		{
			irc_itemQuality = 9;
			Format(str_QualityPrefix, sizeof(str_QualityPrefix), "Customized ");  //Unused?
		}
		case 11:
		{
			irc_itemQuality = 7;
			Format(str_QualityPrefix, sizeof(str_QualityPrefix), "Strange ");
		}
		case 12:
		{
			irc_itemQuality = 9;
			Format(str_QualityPrefix, sizeof(str_QualityPrefix), "Completed ");  //Unused
		}
		case 13:
		{
			irc_itemQuality = 6;
			Format(str_QualityPrefix, sizeof(str_QualityPrefix), "Haunted ");
		}
		case 14:
		{
			irc_itemQuality = 9;
			Format(str_QualityPrefix, sizeof(str_QualityPrefix), "");  //Unused
		}
	}

	// Item name processing
	decl String:tfEnglishName[256];
	decl String:typeName[64];
	decl String:class[64];
	decl String:itemdef[10];
	Format(itemdef, sizeof(itemdef), "%i", iItemIndex);
	if (KvJumpToKey(kv_items, itemdef))	// Jump from items to the item index
	{
		KvGetString(kv_items, "name", tfEnglishName, sizeof(tfEnglishName), "Unknown Item"); // get the item name value from the item index key
		KvGetString(kv_items, "item_slot", typeName, sizeof(typeName), "");
		KvGetString(kv_items, "item_class", class, sizeof(class), "");
		KvGoBack(kv_items);	// jump back up from the item index key to the items root key
	}
	else	// no key found for that item index
		Format(tfEnglishName, sizeof(tfEnglishName), "Unknown Item");

	//Fix up item names
	if (itemQuality != 6)
		ReplaceString(tfEnglishName, sizeof(tfEnglishName), "The ", "", false);
	else if (StrEqual(tfEnglishName, "Craft Bar Level 1", false))
		Format(tfEnglishName, sizeof(tfEnglishName), "Scrap Metal");
	else if (StrEqual(tfEnglishName, "Craft Bar Level 2", false))
		Format(tfEnglishName, sizeof(tfEnglishName), "Reclaimed Metal");
	else if (StrEqual(tfEnglishName, "Craft Bar Level 3", false))
		Format(tfEnglishName, sizeof(tfEnglishName), "Refined Metal");
	else if (StrEqual(tfEnglishName, "Decoder Ring", false))
		Format(tfEnglishName, sizeof(tfEnglishName), "Mann Co. Supply Crate Key");
	else if (StrContains(tfEnglishName, "Supply Crate", false) != -1)
		Format(tfEnglishName, sizeof(tfEnglishName), "Mann Co. Supply Crate");


	// End item name processing

	if (iMethod == -1)
		iMethod = 0;

	switch (iMethod)
	{
		case 0:
		{
			Format(message, sizeof(message), "\x03has found \x03%02d[%s%s]\x03", irc_itemQuality, str_QualityPrefix, tfEnglishName);
		}
		case 1:
		{
			Format(message, sizeof(message), "\x03has crafted \x03%02d[%s%s]\x03", irc_itemQuality, str_QualityPrefix, tfEnglishName);
		}
		case 2:
		{
			Format(message, sizeof(message), "\x03has traded for \x03%02d[%s%s]\x03", irc_itemQuality, str_QualityPrefix, tfEnglishName);
		}
		case 3:
		{
			Format(message, sizeof(message), "\x03has purchased \x03%02d[%s%s]\x03", irc_itemQuality, str_QualityPrefix, tfEnglishName);
		}
		case 4:
		{
			Format(message, sizeof(message), "\x03has unboxed \x03%02d[%s%s]\x03", irc_itemQuality, str_QualityPrefix, tfEnglishName);
		}
		case 5:
		{
			Format(message, sizeof(message), "\x03has been gifted \x03%02d[%s%s]\x03", irc_itemQuality, str_QualityPrefix, tfEnglishName);
		}
		case 8:
		{
			Format(message, sizeof(message), "\x03has earned \x03%02d[%s%s]\x03", irc_itemQuality, str_QualityPrefix, tfEnglishName);
		}
		case 9:
		{
			Format(message, sizeof(message), "\x03has been refunded \x03%02d[%s%s]\x03", irc_itemQuality, str_QualityPrefix, tfEnglishName);
		}
		case 10:
		{
			Format(message, sizeof(message), "\x03has wrapped \x03%02d[%s%s]\x03", irc_itemQuality, str_QualityPrefix, tfEnglishName);
		}
	}

	//Add item details
	StrCat(ircMessage, sizeof(ircMessage), message);


	//Add item level and hat/paint tag (if hat or paint)
	//Format(message, sizeof(message), " \x0303<Level %i>\x03", itemLevel);
	//StrCat(ircMessage, sizeof(ircMessage), message);

	if (StrEqual(typeName, "head", false))
	{
		StrCat(ircMessage, sizeof(ircMessage), " \x0313(Hat!)\x03");
	}
	else if (StrEqual(typeName, "misc", false))
	{
		StrCat(ircMessage, sizeof(ircMessage), " \x0313(Misc Slot Hat!)\x03");
	}
	else if (StrContains(tfEnglishName, "Paint", false) != -1)
	{
		StrCat(ircMessage, sizeof(ircMessage), " \x0307(Paint!)\x03");
	}
	else if (StrEqual(class, "tool", false))
	{
		StrCat(ircMessage, sizeof(ircMessage), " \x0307(Tool!)\x03");
	}

	if (GetConVarBool(v_Debug))
	{
		Format(message, sizeof(message), "\x03  Debug: item def:%s", itemdef);
		StrCat(ircMessage, sizeof(ircMessage), message);
	}

	if (GetConVarBool(v_Timestamp))
	{
		FormatTime(message, sizeof(message), " %x - %X", GetTime()+GetConVarInt(v_Timeoffset))
		StrCat(ircMessage, sizeof(ircMessage), message);
	}

	//----

	if (GetConVarBool(v_ShowSteamID))
	{
		decl String:SteamID[64];
		GetClientAuthString(client, SteamID, sizeof(SteamID));
		Format(SteamID, sizeof(SteamID), " [%s]", SteamID);
		StrCat(ircMessage, sizeof(ircMessage), SteamID);
	}
	IRC_MsgFlaggedChannels("items", ircMessage);
}


public OnPluginEnd()
{
	IRC_CleanUp();
	CloseHandle(kv_items);
}
